package fr.paris.lutece.plugins.form.service;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import fr.paris.lutece.plugins.form.business.FormSubmit;
import fr.paris.lutece.plugins.form.business.validator.IValidator;

/**
 * ValidatorService
 */
public class ValidatorService
{
	private static ValidatorService _singleton;
	private Map<String, IValidator> _mapValidator;
	
	/**
	 * Creates a new instance of ValidatorService
	 */
    private ValidatorService(  )
    {
    	_mapValidator = new HashMap<String, IValidator>(  );
    }
	
    /**
     * Returns the unique instance of ValidatorService
     * @return the unique instance of ValidatorService
     */
	public static ValidatorService getInstance(  )
    {
        if ( _singleton == null )
        {
            _singleton = new ValidatorService(  );
        }

        return _singleton;
    }
	
	/**
	 * Registers a validator
	 * @param strKey the validator key
	 * @param validator the validator
	 */
	public void register( String strKey, IValidator validator )
	{
		_mapValidator.put( strKey, validator );
	}
	
	/**
	 * Unregisters a validator
	 * @param strKey the validator key
	 */
	public void unregister( String strKey )
	{
		_mapValidator.remove( strKey );
	}
	
	/**
	 * Returns the validator which the key is specified in parameter
	 * @param strKey the validator key
	 * @return the validator which the key is specified in parameter
	 */
	public IValidator getValidatorByKey( String strKey )
    {
        return _mapValidator.get( strKey );
    }
	
	/**
	 * Returns all validators
	 * @return all validators
	 */
	public Collection<IValidator> getAllValidators(  )
    {
		return _mapValidator.values(  );
    }
	
	/**
	 * Validates the form using all validators
	 * @param request {@link HttpServletRequest}
	 * @param formSubmit the form submit
	 */
	public void validateForm( HttpServletRequest request, FormSubmit formSubmit )
	{
		for( IValidator validator : getAllValidators(  ) )
        {
        	validator.validateForm( request, formSubmit );
        }
	}
	
	/**
	 * Removes the associations between all validators and the form
	 * @param nIdForm the form id
	 */
	public void removeAssociationsWithForm( int nIdForm )
    {
        for( IValidator validator : getAllValidators(  ) )
        {
        	validator.removeAssociationsWithForm( nIdForm );
        }
    }
}
